
###
###  Copyright 2000-2007 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  etc_passwd.pm - psgconf action to generate /etc/passwd
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Action::GenerateFile::etc_passwd;

use strict;

use PSGConf::Action::GenerateFile;

our @ISA = qw(PSGConf::Action::GenerateFile);


###############################################################################
###  generate() method
###############################################################################

sub generate
{
	my ($self, $fh, $psgconf) = @_;
	my ($entry, $sort_func);

	$sort_func = $self->{sort_func};

	foreach $entry (sort $sort_func values %{$self->{user_info}})
	{
		if ( defined $entry->{name} )
		{
			print $fh $entry->{name} . ':';
			if ($self->{add_passwords})
			{
				print $fh (defined($entry->{passwd})
					? $entry->{passwd}
					: '*') . ':';
			}
			else
			{
				print $fh $self->{passwd_token} . ':';
			}
			print $fh $entry->{uid} . ':';
			print $fh $entry->{gid} . ':';
			print $fh $entry->{gecos} . ':';
			print $fh $entry->{home} . ':';
			print $fh $entry->{shell};
			print $fh "\n";
		}
		else 
		{
			warn "\tWARNING!!! login has no name, uid => (" .
				$entry->{uid} . ")\n";
		}
	}

	return 1;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::etc_passwd - generate /etc/passwd file

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::etc_passwd;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::etc_passwd->new(
			'name'		=> '/etc/passwd',
			'user_info'	=> {
				'username'	=> { ...  },
				...
			},
			'sort_func'	=> \&_sort_func,
			'passwd_token'	=> '*',
			'add_passwords'	=> 0,
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::etc_passwd> module provides a B<PSGConf>
action class for generating the F</etc/passwd> file.

The B<PSGConf::Action::GenerateFile::etc_passwd> class is derived from the
B<PSGConf::Action::GenerateFile> class, but it defines/overrides the
following methods:

=over 4

=item generate()

Generates the F</etc/passwd> file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::etc_passwd> class supports the following
attributes:

=over 4

=item I<user_info>

A hash mapping user names to a hash containing the fields for that user.
The hash for each user should contain the fields I<name>, which is set
to the name of the user; I<uid>, which is set to the numeric uid of the
user; I<gid>, which is set to the numeric gid of the user's primary
group; I<gecos>, which is set to the user's GECOS information; I<home>,
which is set to the user's home directory; and I<shell>, which is set to
the user's login shell.

=item I<sort_func>

A reference to a subroutine that will be used to sort the entries in
I<user_info>.

=item I<passwd_token>

A string containing the token to be used in the password field for users
that do not have a password.

=item I<add_passwords>

A boolean value indicating whether users' passwords should be added to
the F</etc/passwd> file.

=back

=head1 AUTHOR

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut


