
###
###  Copyright 2000-2007 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PSGConf::Action::GenerateFile::etc_profile - action to generate /etc/profile
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Action::GenerateFile::etc_profile;

use strict;

use PSGConf::Action::GenerateFile;

our @ISA = qw(PSGConf::Action::GenerateFile);


###############################################################################
###  generate method
###############################################################################

sub generate
{
	my ($self, $fh, $psgconf) = @_;

	print $fh <<'EOF';

###
### set environment variables
###

EOF
	print $fh 'PATH='
		  . join(':', @{$self->{path}})
		  . "\nexport PATH\n\n"
		if (exists($self->{path}));
	print $fh 'MANPATH='
		  . join(':', @{$self->{manpath}})
		  . "\nexport MANPATH\n\n"
		if (exists($self->{manpath}));
	map {
		print $fh "$_=\"$self->{env_vars}->{$_}\"\nexport $_\n";
	} sort keys %{$self->{env_vars}};

	if (defined($self->{shell_aliases}))
	{
		print $fh <<'EOF';
   
###  
### set shell aliases
###

EOF
		my ($cmd);
		map {
			$cmd = $self->{shell_aliases}->{$_};
			if ($cmd =~ m/\<ARGS\>/)
			{
				### translate "<ARGS>" to "$*"
				$cmd =~ s/\<ARGS\>/\$\*/g;
          
				print $fh "function $_ {\n\t";
				print $fh $cmd . "\n";
				print $fh "}\n";
			}
			else
			{
				print $fh "alias $_=\"$self->{shell_aliases}->{$_}\"\n";
			}
		} (sort keys %{$self->{shell_aliases}});
	}

	if (defined($self->{commands}))
	{
		print $fh <<'EOF';


###
### other commands
###

EOF
		print $fh "\n$self->{commands}\n";
	}

	print $fh "\n\n";

	return 1;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::etc_profile - generate /etc/profile file

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::etc_profile;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::etc_profile->new(
			'name'		=> '/etc/profile',
			'description'	=> 'sh and ksh startup file',
			'path'		=> [ ... ],
			'manpath'	=> [ ... ],
			'env_vars'	=> { ... },
			'shell_aliases'	=> { ... },
			'commands'	=> '...',
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::etc_profile> module provides a
B<PSGConf> action class for generating the F</etc/profile> file.

The B<PSGConf::Action::GenerateFile::etc_profile> class is derived from the
B<PSGConf::Action::GenerateFile> class, but it defines/overrides the
following methods:

=over 4

=item generate()

Creates the F</etc/profile> file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::etc_profile> class supports the following
attributes:

=over 4

=item I<path>

A list containing directories to be added to the C<PATH> environment
variable.

=item I<manpath>

A list containing directories to be added to the C<MANPATH> environment
variable.

=item I<env_vars>

A hash mapping variables to their values.

=item I<shell_aliases>

A hash containing shell aliases to define.  In the value of aliases,
the string C<E<lt>ARGSE<gt>> will be replaced with C<$*>.

=item I<commands>

A string containing commands to be added to the F</etc/profile> file.

=back

=head1 SEE ALSO

sh(1)

ksh(1)

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut

