
###
###  Copyright 2000-2007 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  inetd_conf.pm - psgconf action class for generating inetd.conf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Action::GenerateFile::inetd_conf;

use strict;

use File::Basename;

use PSGConf::Action::GenerateFile;

our @ISA = qw(PSGConf::Action::GenerateFile);


###############################################################################
###  generate() method
###############################################################################

my %_socket_types = (
	tcp	=> 'stream',
	udp	=> 'dgram',
);

my %_proto_wait = (
	tcp	=> 0,
	udp	=> 1
);

sub generate
{
	my ($self, $fh, $psgconf) = @_;
	my ($label, $entry, $svc, $proto, $ver);

	foreach $label (sort keys %{$self->{services}})
	{
		$entry = $self->{services}->{$label};
		($svc, $proto) = split(/\//, $label);

		$entry->{socket_type} = $_socket_types{$proto}
			if (!defined($entry->{socket_type}));
		$entry->{wait} = $_proto_wait{$proto}
			if (!defined($entry->{wait}));
		$entry->{user} = 'root'
			if (!defined($entry->{user}));
		$entry->{server_args} = basename($entry->{server})
					. ' ' . $entry->{server_args}
			if ($entry->{server} ne 'internal'
			    && !exists($entry->{flags}->{NAMEINARGS}));

		### some platforms use "tcp6" and "udp6" for IPv6
		$proto .= '6'
			if (exists($entry->{flags}->{IPv6})
			    && $proto =~ m/^(tcp|udp)$/
			    && $psgconf->data_obj('platform')->match('aix|solaris|freebsd|openbsd'));

		printf $fh "%-15s %s\t%s\t%s\t%s\t%s\t%s\n",
			   $svc,
			   $entry->{socket_type},
			   $proto,
			   ($entry->{wait} =~ m/\D/
			    ? $entry->{wait}
			    : ($entry->{wait} ? 'wait' : 'nowait')),
			   $entry->{user},
			   $entry->{server},
			   $entry->{server_args};
	}

	foreach $label (sort keys %{$self->{rpc_services}})
	{
		$entry = $self->{rpc_services}->{$label};
		($svc, $ver) = split(/\//, $label);

		$entry->{user} = 'root'
			if (!defined($entry->{user}));
		$entry->{server_args} = basename($entry->{server})
					. ' ' .  (( exists $entry->{server_args} )?
								$entry->{server_args} : '')
			if (!exists($entry->{flags}->{NAMEINARGS}));

		if ($psgconf->data_obj('platform')->match('aix'))
		{
			$entry->{socket_type} = 'sunrpc_' . $entry->{protocol}
				if (!defined($entry->{socket_type}));

			printf $fh "%-15s %s\t%s\t%s\t%s\t%s\t%s\n",
				   $svc,
				   $entry->{socket_type},
				   $entry->{protocol},
				   ($entry->{wait} ? 'wait' : 'nowait'),
				   $entry->{user},
				   $entry->{server},
				   $entry->{server_args};
			next;
		}

		if ($psgconf->data_obj('platform')->match('hpux'))
		{
			printf $fh "%-15s %s\t%s\t%s\t%s\t%s\t%d %d\t%s\n",
				   'rpc',
				   $entry->{socket_type},
				   $entry->{protocol},
				   ($entry->{wait} =~ m/\D/
				    ? $entry->{wait}
				    : ($entry->{wait} ? 'wait' : 'nowait')),
				   $entry->{user},
				   $entry->{server},
				   $entry->{rpc_number},
				   $ver,
				   $entry->{server_args};
			next;
		}

		### this should work everywhere else
		### (tested under Solaris, IRIX, and OpenBSD)
		printf $fh "%-15s %s\trpc/%s\t%s\t%s\t%s\t%s\n",
			   $label,
			   $entry->{socket_type},
			   $entry->{protocol},
			   ($entry->{wait} =~ m/\D/
			    ? $entry->{wait}
			    : ($entry->{wait} ? 'wait' : 'nowait')),
			   $entry->{user},
			   $entry->{server},
			   $entry->{server_args};
	}

	print $fh $self->{literal_text} . "\n"
		if (defined($self->{literal_text}));

	return 1;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::inetd_conf - generate inetd.conf files

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::inetd_conf;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::inetd_conf->new(
			name		=> '/etc/inetd.conf',
			services	=> {
			   	'echo/udp' => {
					server => 'internal'
				},
				'ftp/tcp' => {
					server => '/usr/local/sbin/in.ftpd',
					server_args => '-a -d -l'
				},
				...
			},
			rpc_services	=> {
				'rquotad/1' => {
					socket_type => 'tli',
					protocol => 'datagram_v',
					wait => 1,
					server => '/usr/lib/nfs/rquotad'
				},
				...
			},
			literal_text	=> '...',
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::inetd_conf> module provides a
B<PSGConf> action class for generating an F<inetd.conf> file.

The B<PSGConf::Action::GenerateFile::inetd_conf> class is derived from the
B<PSGConf::Action::GenerateFile> class, but it defines/overrides the
following methods:

=over 4

=item generate()

Generates the F<inetd.conf> file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::inetd_conf> class supports the following
attributes:

=over 4

=item I<services>

A hash mapping containing entries for F<inetd.conf>.  The hash keys are
strings of the form I<port>/I<protocol> (e.g., C<ftp/tcp>).  The
hash value is an anonymous hash containing the service's attributes.
The following attributes are known:

=over 4

=item I<socket_type>

The socket type.  If not present, it defaults to C<stream> for the C<tcp>
protocol or C<dgram> for the C<udp> protocol; for all other protocols,
this attribute must be specified.

=item I<wait>

A boolean value that indicates whether a new server should be forked for
each new request.  If not present, it defaults to true for the C<udp>
protocol or false for the C<tcp> protocol; for all other protocols, this
attribute must be specified.

(If set to a string rather than a boolean value, the string is written
to the appropriate field of the F<inetd.conf> file.  This is ugly, but
may be needed to support vendor-specific extensions such as AIX's C<SRC>
option, HP-UX's C<swait> option, or OpenBSD's C<.max> suffix.)

=item I<user>

The user that the service should run as.  If not specified, defaults to
C<root>.

=item I<server>

The absolute path to the server program.  This field is required.

=item I<server_args>

Arguments to be passed to the server program.  Note that the value must
not include the program name itself unless the I<NAMEINARGS> flag is set
(see below).

=item I<flags>

A hash whose keys are a set of boolean flags that modify the
service.  Only two flags are supported:

=over 4

=item I<IPv6>

Service supports IPv6.

=item I<NAMEINARGS>

The I<server_args> field includes the program name itself.

=back

=back

=item I<rpc_services>

A hash mapping containing RPC entries for F<inetd.conf>.  The hash
keys are strings of the form I<rpc_service>/I<rpc_version> (e.g.,
C<rquotad/1>).  The hash value is an anonymous hash containing the
service's attributes.  The attributes are identical to those for the
I<services> hash, with the following exceptions:

=over 4

=item *

The I<socket_type> attribute is mandatory on all platforms except for
AIX.

=item *

The I<wait> attribute is mandatory.

=item *

The I<protocol> attribute must be set to the service's protocol.

=back

=item I<literal_text>

Literal text to be appended to the F<inetd.conf> file.

=back

=head1 BUGS

The I<literal_text> attribute is an ugly hack and may be removed in a
future release.

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut


