
###
###  Copyright 2000-2007 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  sendmail_mc.pm - psgconf action to generate sendmail .mc files
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Action::GenerateFile::sendmail_mc;

use strict;

use PSGConf::Action::GenerateFile;

our @ISA = qw(PSGConf::Action::GenerateFile);


###############################################################################
###  generate() method
###############################################################################

sub _print_feature
{
	my ($self, $fh, $feature, $args) = @_;

	print $fh "FEATURE(`$feature'";
	if (defined($args))
	{
		$args =~ s/(.*)/`$1'/
			if ($args !~ m/`.*'/);
		print $fh ", $args";
	}
	print $fh ")dnl\n";
}


sub generate
{
	my ($self, $fh, $psgconf) = @_;
	my ($smopt, $tmp, $qgrp);

	print $fh "VERSIONID(`$self->{versionid}')dnl\n"
		if (defined($self->{versionid}));

	print $fh "OSTYPE(`$self->{ostype}')dnl\n"
		if (defined($self->{ostype}));

	print $fh "\n";

	foreach $smopt (sort keys %{$self->{defines}})
	{
		print $fh "define(`$smopt'";
		if (defined($self->{defines}->{$smopt}))
		{
			$tmp = $self->{defines}->{$smopt};
			$tmp =~ s/(.*)/`$1'/
				if ($tmp !~ m/`.*'/);
			print $fh ", $tmp";
		}
		print $fh ")dnl\n";
	}

	print $fh "\n";

	foreach $smopt (sort keys %{$self->{features}})
	{
		if (ref($self->{features}->{$smopt}) eq 'ARRAY')
		{
			map {
				$self->_print_feature($fh, $smopt, $_);
			} (@{$self->{features}->{$smopt}});
		}
		else
		{
			$self->_print_feature($fh, $smopt,
					      $self->{features}->{$smopt});
		}
	}

	print $fh "MASQUERADE_AS(`$self->{masquerade_as}')dnl\n"
		if (defined($self->{masquerade_as}));

	foreach $smopt (@{$self->{exposed_users}})
	{
		print $fh "EXPOSED_USER(`$smopt')dnl\n",
	}

	print $fh "\n";

	map { print $fh "MAILER(`$_')dnl\n"; } @{$self->{mailers}};

	print $fh "\n"
		if (exists($self->{queue_groups})
		    && ref($self->{queue_groups}) eq 'HASH'
		    && %{$self->{queue_groups}});

	foreach $qgrp (sort keys %{$self->{queue_groups}})
	{
		print $fh "QUEUE_GROUP(`$qgrp', `"
			  . join(', ', map {
				"$_=$self->{queue_groups}->{$qgrp}->{$_}"
			    } (sort keys %{$self->{queue_groups}->{$qgrp}}))
			  . "')dnl\n";
	}

	if (defined($self->{literal}))
	{
		print $fh "\n$self->{literal}\n";
	}

	return 1;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::sendmail_mc - generate sendmail.mc file

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::sendmail_mc;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::sendmail_mc->new(
			'name'		=> '/etc/mail/sendmail.mc',
			'versionid'	=> '...',
			'ostype'	=> '...',
			'defines'	=> { ... },
			'features'	=> { ... },
			'masquerade_as'	=> '...',
			'mailers'	=> [ ... ],
			'queue_groups'	=> { ... },
			'literal'	=> '...',
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::sendmail_mc> module provides a
B<PSGConf> action class for generating the F<sendmail.mc> file.

The B<PSGConf::Action::GenerateFile::sendmail_mc> class is derived from
the B<PSGConf::Action::GenerateFile> class, but it defines/overrides
the following methods:

=over 4

=item generate()

Generates the F<sendmail.mc> file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::sendmail_mc> class supports the following
attributes:

=over 4

=item I<versionid>

The argument to the C<VERSIONID> macro.  This attribute is optional.

=item I<ostype>

The argument to the C<OSTYPE> macro.

=item I<defines>

A hash of macro definitions and their values.  These are defined using
the C<m4> C<define> macro.

=item I<features>

A hash of feature definitions and optional values.  These are set using
the C<FEATURE> macro.

=item I<masquerade_as>

A hostname to masquerade as.

=item I<mailers>

A list of mailers for which C<MAILER> macros are added.

=item I<queue_groups>

A hash of queue groups to define.  The key is the name of the queue
group, and the value is an anonymous hash containing the attributes of
the queue group.

=item I<literal>

Literal text to be added to the end of the file.

=back

=head1 BUGS

The I<literal> attribute is an ugly hack.

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut


