
###
###  Copyright 2000-2007 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  xinetd_conf.pm - psgconf action class for generating xinetd.conf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Action::GenerateFile::xinetd_conf;

use strict;

use File::Basename;

use PSGConf::Action::GenerateFile;

our @ISA = qw(PSGConf::Action::GenerateFile);


###############################################################################
###  generate() method
###############################################################################

my %_socket_types = (
        tcp	=> 'stream',
	udp	=> 'dgram'
);

my %_proto_wait = (
        tcp	=> 0,
        udp	=> 1
);

sub generate
{
	my ($self, $fh, $psgconf) = @_;
	my ($key, $entry, $svc, $proto, $ver, $id, %ids);

	if (%{$self->{defaults}})
	{
		print $fh "\ndefaults\n{\n";
		map {
			print $fh "\t$_\t= $self->{defaults}->{$_}\n";
		} (sort keys %{$self->{defaults}});
		print $fh "}\n";
	}

	foreach $key (sort keys %{$self->{services}})
	{
		my (@types);

		$entry = $self->{services}->{$key};
		($svc, $proto) = split('/', $key);

		$entry->{socket_type} = $_socket_types{$proto}
			if (!defined($entry->{socket_type}));
		$entry->{wait} = $_proto_wait{$proto}
			if (!defined($entry->{wait}));
		$entry->{user} = 'root'
			if (!defined($entry->{user}));
		if ($entry->{server} eq 'internal')
		{
			push(@types, 'INTERNAL');
			delete($entry->{server});
			delete($entry->{server_args});
			delete($entry->{user});
		}

		### FIXME: check whether we need to add "UNLISTED" to @types
		### (if so, port attribute must be present as well)

		$id = $svc . '-' . $entry->{socket_type};
		if (exists($ids{$id}))
		{
			warn "\n\t!!! WARNING : skipping duplicate entry for '$id'\n";
			next;
		}
		$ids{$id} = undef;

		### now we actually write the entry
		print $fh "\nservice $svc\n{\n";
		printf $fh "\t%-15s = %s\n", 'id', $id;

		### add "type" attribute
		printf $fh ("\t%-15s = %s\n", 'type', join(' ', @types))
			if (@types);

		### add "protocol" attribute
		printf $fh "\t%-15s = %s\n", 'protocol', $proto;

		### add "wait" attribute
		if ($entry->{wait} eq 'yes' || $entry->{wait} > 0) {
			printf $fh "\t%-15s = %s\n", 'wait', 'yes';
		} elsif ($entry->{wait} eq 'no' || !$entry->{wait}) {
			printf $fh "\t%-15s = %s\n", 'wait', 'no';
		}

		### add remaining attributes
		map {
			printf $fh "\t%-15s = %s\n",
				   $_,
				   (ref($entry->{$_}) eq 'HASH'
				    ? join(' ', sort keys %{$entry->{$_}})
				    : $entry->{$_});
		} (sort grep { $_ !~ m/^(protocol|wait|type|id)$/ } keys %$entry);

		print $fh "}\n\n";
	}

	foreach $key (sort keys %{$self->{rpc_services}})
	{
		my (@types) = qw(RPC);

		$entry = $self->{rpc_services}->{$key};
		($svc, $ver) = split('/', $key);

		$entry->{socket_type} = $_socket_types{$proto}
			if (!defined($entry->{socket_type}));
		$entry->{wait} = $_proto_wait{$proto}
			if (!defined($entry->{wait}));
		$entry->{user} = 'root'
			if (!defined($entry->{user}));

		### FIXME: check whether we need to add "UNLISTED" to @types

		$id = $svc . '-rpc';
		if (exists($ids{$id}))
		{
			warn "\n\t!!! WARNING : skipping duplicate entry for '$id'\n";
			next;
		}
		$ids{$id} = undef;

		### now we actually write the entry
		print $fh "\nservice $svc\n{\n";
		printf $fh "\t%-15s = %s\n", 'id', $id;

		### add "type" attribute
		printf $fh ("\t%-15s = %s\n", 'type', join(' ', @types))
			if (@types);

		### add "rpc_version" attribute
		printf $fh "\t%-15s = %s\n", 'rpc_version', $ver;

		### add "wait" attribute
		printf $fh "\t%-15s = %s\n", 'wait',
			   ($entry->{wait} ? 'yes' : 'no');

		### add remaining attributes
		map {
			printf $fh "\t%-15s = %s\n",
				   $_,
				   (ref($entry->{$_}) eq 'HASH'
				    ? join(' ', sort keys %{$entry->{$_}})
				    : $entry->{$_});
		} (sort grep { $_ !~ m/^(wait|type|id)$/ } keys %$entry);

		print $fh "}\n\n";
	}

	print $fh $self->{literal_text} . "\n"
		if (defined($self->{literal_text}));

	return 1;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::xinetd_conf - generate xinetd.conf file

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::xinetd_conf;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::xinetd_conf->new(
			name		=> '/etc/xinetd.conf',
			services	=> {
			   	'echo/udp' => {
					server => 'internal'
				},
				'ftp/tcp' => {
					server => '/usr/local/sbin/in.ftpd',
					server_args => '-a -d -l'
				},
				...
			},
			rpc_services	=> {
				'rquotad/1' => {
					socket_type => 'dgram',
					protocol => 'udp',
					wait => 1,
					server => '/usr/lib/nfs/rquotad'
				},
				...
			},
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::xinetd_conf> module provides a
B<PSGConf> action class for generating an F<xinetd.conf> file.

The B<PSGConf::Action::GenerateFile::xinetd_conf> class is derived from the
B<PSGConf::Action::GenerateFile> class, but it defines/overrides the
following methods:

=over 4

=item generate()

Generates the F<xinetd.conf> file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::xinetd_conf> class supports the following
attributes:

=over 4

=item I<services>

A hash mapping containing entries for F<xinetd.conf>.  The hash keys are
strings of the form I<port>/I<protocol> (e.g., C<ftp/tcp>).  The
hash value is an anonymous hash containing the service's attributes.
The attributes are the fields documented in xinetd.conf(5).  Here are
some of the more common attributes:

=over 4

=item I<socket_type>

The socket type.  If not present, it defaults to C<stream> for the C<tcp>
protocol or C<dgram> for the C<udp> protocol.

=item I<wait>

A boolean value that indicates whether a new server should be forked for
each new request.

=item I<user>

The user that the service should run as.  If not specified, defaults to
C<root>.

=item I<server>

The absolute path to the server program.  This field is required.

=item I<server_args>

Arguments to be passed to the server program.  Note that this does not
include the program name itself unless the I<NAMEINARGS> flag is set
(see below).

=item I<flags>

A hash whose keys are a set of boolean flags that modify the
service.  Only two flags are supported:

=over 4

=item I<IPv6>

Service supports IPv6.

=item I<NAMEINARGS>

The I<server_args> field includes the program name itself.

=back

=back

=item I<rpc_services>

A hash mapping containing RPC entries for F<xinetd.conf>.  The hash
keys are strings of the form I<rpc_service>/I<rpc_version> (e.g.,
C<rquotad/1>).  The hash value is an anonymous hash containing the
service's attributes.  The attributes are identical to those for the
I<services> hash, with the following exceptions:

=over 4

=item *

The I<socket_type> attribute is mandatory on all platforms except for
AIX.

=item *

The I<wait> attribute is mandatory.

=item *

The I<protocol> attribute must be set to the service's protocol.

=back

=back

=head1 BUGS

The I<type> attribute is set automatically.  However, there is currently
no support for the C<UNLISTED> type.

=head1 SEE ALSO

xinetd.conf(5)

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut


