
###
###  Copyright 2000-2004 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PSGConf::Action::PackageManager::FreeBSD::Packages - FreeBSD pkg action type for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Action::PackageManager::FreeBSD::Packages;

use strict;

use PSGConf::Action::PackageManager::FTPArchive;
use PSGConf::Action::PackageManager::FreeBSD;
use PSGConf::Data::String;
use PSGConf::Util;

use File::Basename;

our @ISA = qw(PSGConf::Action::PackageManager::FreeBSD
			PSGConf::Action::PackageManager::FTPArchive);

###############################################################################
###  configuration constructor
###############################################################################

sub conf
{
     my ($class, $psgconf) = @_;

     PSGConf::Action::PackageManager::FTPArchive->conf($psgconf);
     PSGConf::Action::PackageManager::FreeBSD->conf($psgconf);
}

###############################################################################
###  pkg_get_latest_version() method
###############################################################################

sub pkg_get_latest_version
{
     my ($self, $psgconf, $pkg) = @_;

     return $self->get_latest_version (
          $psgconf,
          $pkg,
		$psgconf->data_obj('env_vars')->find('PACKAGESITE'),
          qr{(\Q${pkg}\E\-[^-]+).tbz}
          );
}

###############################################################################
###  pkg_get_dependencies() method
###############################################################################

sub pkg_get_dependencies
{
     my ($self, $psgconf, $pkg) = @_;
     my (@pkgs, @files, $file);

     # Need to escape the + and . in the package name, hence \Q and \E
     my ($regex) = "\Q${pkg}\E.tbz";

     # First see if the package is already locally installed
     if ( -d $psgconf->data_obj('pkg_db_dir')->get() . '/' . $pkg ) {  
          @pkgs = $self->pkg_get_installed_dependencies($psgconf, $pkg);

     # find the package on the server
	}  else {
          @files = $self->list_files_from_url($psgconf,
					$psgconf->data_obj('env_vars')->find('PACKAGESITE'),
					qr{$regex});

          # we should have only found one entry
          if ( scalar @files == 1 ) {

               # See if we have the package tarball 'cached'.
               $file = &basename ($files[0]);
			$file = $self->get_file_from_url($psgconf, $files[0])
				if ( ! -f "$psgconf->{tmpdir}/$file" );

               if ( -f $file ) {
          
                    @pkgs = $self->_extract_deps(
					&_unpack_pkg ($psgconf, $pkg, $file));
                         
                    # Remove the package and directory.
                    warn "\t!!! unlink($file): $!\n"
                         if ( $psgconf->{rm_tmpfiles}
                              && !rmtree ($pkg) );

               }
          } elsif ( ! scalar @files ) {
               warn "\tCould not find URL for ($pkg)\n";
		} else {
               warn "\tFound too many URLs for ($pkg)\n";
          }
     }

     return @pkgs;
}

sub _unpack_pkg
{
	my ($psgconf, $pkg, $file) = @_;
}

###############################################################################
###  pkg_install() method
###############################################################################

sub pkg_install
{
	my ($self, $psgconf, $pkg) = @_;
	my ($err, $cmd);

	$cmd = $psgconf->data_obj('pkg_cmd_path')->get() . 'pkg_add -r';
     $cmd .= 'v'
          if ($psgconf->{verbose});
	$cmd .= ' ' . $pkg;

     $err=&PSGConf::Util::RunCommand($cmd);

     print "\n"
          if ($psgconf->{verbose});

	return $err;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::PackageManager::FreeBSD::Packages - install FreeBSD packages

=head1 SYNOPSIS

  use PSGConf::Action::PackageManager::FreeBSD::Packages;

  $psgconf->register_actions(
		PSGConf::Action::PackageManager::FreeBSD::Packages->new(
			'quiet'		=> 1
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::PackageManager::FreeBSD::Packages> module provides 
a B<PSGConf> action class for updating FreeBSD packages.

The B<PSGConf::Action::PackageManager::FreeBSD::Packages> class is derived 
from the B<PSGConf::Action::PackageManager::FreeBSD> class, but it 
defines/overrides the following methods:

=over 4

=item I<pkg_get_latest_version($pkg)>

A method that will return a B<PSGConf::Data::String> that contains
the latest version of $pkg from the package repository.  Called from the 
B<PSGConf::Action::PackageManager::check()> method.  It uses the
environment variable B<PACKAGESITE> to determine where to look
for these new packages from.

=item I<pkg_get_dependencies($pkg)>

A method that will return a B<PSGConf::Data::List> that contains
all the packages that $pkg will depend upon, regardless
if they have been installed or not.   This is for the package on
the package repository and $pkg will have the version in it as well.
Called from the B<PSGConf::Action::PackageManager::diff()> method.

=item I<pkg_install($pkg)>

A method that will install a $pkg on the system.  Returns 0 for a
successfull install, non-zero for a failure.  Called from 
B<PSGConf::Action::PackageManager::do()> method.  It uses the
environment variable B<PACKAGESITE> to determine where to get
these packages from (see pkg_add(1) man page for details).

=back

=head1 SEE ALSO

C<CE<lt>pkg_add(1)E<gt>>

C<CE<lt>pkg_delete(1)E<gt>>

C<CE<lt>pkg_version(1)E<gt>>

L<perl>

L<PSGConf>

L<PSGConf::Action>

L<PSGConf::Action::PackageManageru::FreeBSD>

L<PSGConf::Action::PackageManageru::FTPArchive>

L<PSGConf::Data::String>

L<PSGConf::Util>

L<File::Basename>

=cut

