
###
###  Copyright 2000-2004 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PSGConf::Action::PackageManager::FreeBSD::Ports - FreeBSD ports action type for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Action::PackageManager::FreeBSD::Ports;

use strict;

use PSGConf::Action::PackageManager::FreeBSD;
use PSGConf::Data::String;
use PSGConf::Util;

use Cwd;

our @ISA = qw(PSGConf::Action::PackageManager::FreeBSD);

our $pkg_ports_index;

###############################################################################
###  configuration constructor
###############################################################################

sub conf
{
     my ($class, $psgconf) = @_;
	my ($index);

     PSGConf::Action::PackageManager::FreeBSD->conf($psgconf);

	$index = '/usr/ports/INDEX';

	$index .= ($psgconf->data_obj('platform')->match('freebsd6'))
			? '-6'
			: ($psgconf->data_obj('platform')->match('freebsd5'))
				? '-5'
				: '';

     $psgconf->register_data(
          pkg_index_file     => PSGConf::Data::String->new(
                              value => $index
                         ),
		pkg_port_umask   => PSGConf::Data::Integer->new(
						value => 022
					),
          pkg_make_cmd     => PSGConf::Data::String->new(
                              value => "/usr/bin/make"
                         )
	);
}


###############################################################################
###  pkg_get_latest_version() method
###############################################################################

sub pkg_get_latest_version
{
	my ($self, $psgconf, $pkg) = @_;
	my ($pkg2, $key, $key2, $regex);
	my ($version) = '';

	$regex = '^' . $pkg . $self->{seperator}->[0];
	$regex .= '[^' . $self->{seperator}->[0] . ']*$';

	$self->_load_index($psgconf) if (!$pkg_ports_index);

	if ( defined $pkg_ports_index->{$pkg}->[0] ) {
		($pkg) = $self->Parse($pkg_ports_index->{$pkg}->[0]);
		$version = $pkg_ports_index->{$pkg}->[0];
		$version =~ s/^\Q${pkg}\E$self->{seperator}->[0]//;
	} else {
		foreach $key (sort keys %{$pkg_ports_index}) {
			if ($key =~ /$regex/) {
				($pkg2) = $self->Parse($pkg_ports_index->{$key}->[0]);
				$version = $pkg_ports_index->{$key}->[0];
				$version =~ s/^\Q${pkg}\E$self->{seperator}->[0]//;
				$key2 = $key
					if ( $pkg2 eq $pkg );
			}
		}

		if ( defined $pkg_ports_index->{$key2}->[0] ) {
			($pkg) = $self->Parse($pkg_ports_index->{$key2}->[0]);
			$version = $pkg_ports_index->{$key2}->[0];
			$version =~ s/^\Q${pkg}\E$self->{seperator}->[0]//;
		} elsif ( $psgconf->{verbose} ) {
     		warn "\t* no updates found for package $pkg\n";
		}
	}

	return $version;
}

sub _load_index
{
	my ($self, $psgconf) = @_;
	my (@fields, $pkg);
	local *FP;

	if (open (FP, "<" . $psgconf->data_obj('pkg_index_file')->get())) {
		while (<FP>) {
			chomp;
			@fields = split(/\|/o);
			($pkg) = $self->Parse($fields[0]);

			if ( ! defined $pkg_ports_index->{$pkg} ) {
				@{$pkg_ports_index->{$pkg}} = @fields;
			} else {

				###
				### OK, we found a collision and need to deal with it.
				### So remove the old version, and put it back in the
				### list with its full version as the hash key.
				###

				my ($key) = $pkg_ports_index->{$pkg}->[0];
				@{$pkg_ports_index->{$key}} = @{$pkg_ports_index->{$pkg}};

				delete $pkg_ports_index->{$pkg};

				@{$pkg_ports_index->{$fields[0]}} = @fields;
			}
		}
		close FP;

	} else {
          warn "\t!!! Could not open " . $psgconf->data_obj('pkg_index_file')->get() . ": $!\n";
	}
}

###############################################################################
###  pkg_get_dependencies() method
###############################################################################

sub pkg_get_dependencies
{
	my ($self, $psgconf, $pkg) = @_;
	my (@pkgs);

	$self->_load_index($psgconf) if (!$pkg_ports_index);

	my ($pkg2) = $self->Parse($pkg);

	if ( defined $pkg_ports_index->{$pkg} ) {
		@pkgs = split (/[\s]+/o, $pkg_ports_index->{$pkg}->[8]);
	} elsif ( defined $pkg_ports_index->{$pkg2} ) {
		@pkgs = split (/[\s]+/o, $pkg_ports_index->{$pkg2}->[8]);
     } elsif ( -d $psgconf->data_obj('pkg_db_dir')->get() . '/' . $pkg ) {  
		@pkgs = $self->pkg_get_installed_dependencies($psgconf, $pkg);
	} else {
		warn "\tCould not find dependencies for ($pkg)\n"; 
	}

	return @pkgs;
}

###############################################################################
###  pkg_install() method
###############################################################################

sub pkg_install
{
	my ($self, $psgconf, $pkg) = @_;
	my ($err, $cmd, $pwd, $pkg2, $umask);

	$pwd = getcwd;

	($pkg2) = $self->Parse($pkg);

	if ( -d $pkg_ports_index->{$pkg}->[1] ) {
		chdir ($pkg_ports_index->{$pkg}->[1]);
	} elsif ( -d $pkg_ports_index->{$pkg2}->[1] ) {
		chdir ($pkg_ports_index->{$pkg2}->[1]);
	} else {
		warn "\tCannot find port build directory for package ($pkg)\n";
		return -1;
	}

	### Force the umask to be something that everyone will be
	### able to read and execute the files that are being installed
	### and not rely on the default enviroment to set.
	$umask = umask $psgconf->data_obj('pkg_port_umask')->get();

	$cmd = $psgconf->data_obj('pkg_make_cmd')->get() . " -DBATCH -DFORCE_PKG_REGISTER all install clean";

     $cmd .= '> /dev/null'
          if ( ! $psgconf->{verbose} );

     $err=&PSGConf::Util::RunCommand($cmd);

	### Be nice and reset the umask back to what the Administrator had
	umask $umask;

     print "\n"
          if ($psgconf->{verbose});

	chdir ($pwd);

	return $err;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::PackageManager::FreeBSD::Ports - install FreeBSD ports

=head1 SYNOPSIS

  use PSGConf::Action::PackageManager::FreeBSD::Ports;

  $psgconf->register_actions(
		PSGConf::Action::PackageManager::FreeBSD::Ports->new(
			'quiet'		=> 1
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::PackageManager::FreeBSD::Ports> module provides 
a B<PSGConf> action class for updating FreeBSD ports.

The B<PSGConf::Action::PackageManager::FreeBSD::Ports> class is derived
from the B<PSGConf::Action::PackageManager::FreeBSD> class, but it
defines/overrides the following methods:

=over 4

=item I<pkg_get_latest_version($pkg)>

A method that will return a B<PSGConf::Data::String> that contains
the latest version of $pkg from the package repository.  Called from the 
B<PSGConf::Action::PackageManager::check()> method.

=item I<pkg_get_dependencies($pkg)>

A method that will return a B<PSGConf::Data::List> that contains
all the packages that $pkg will depend upon, regardless
if they have been installed or not.   This is for the package on
the package repository and $pkg will have the version in it as well.
Called from the B<PSGConf::Action::PackageManager::diff()> method.

=item I<pkg_install($pkg)>

A method that will install a $pkg on the system.  Returns 0 for a
successfull install, non-zero for a failure.  Called from 
B<PSGConf::Action::PackageManager::do()> method.


=back


The constructor also registers the following data objects:

=over 4

=item I<pkg_index_file>

A B<PSGConf::Data::String> object that determines the location of
the FreeBSD port system index file.  Used to determine the current
version and dependency list for a given package.  The default is 
I</usr/ports/INDEX>.

=item I<pkg_port_umask>

A B<PSGConf::Data::Integer> object that determines the default umask
for the ports system to use.  The default is I<022>.

=item I<pkg_make_cmd>

A B<PSGConf::Data::String> object that determines the location of
the make command.  Defaults to F</usr/bin/make>.

=back


=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action>

L<PSGConf::Action::PackageManager>

L<PSGConf::Data::String>

L<PSGConf::Util>

L<Cwd>

C<CE<lt>pkg_version(1)E<gt>>

=cut

