
###
###  Copyright 2000-2004 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PSGConf::Action::PackageManager::RedHat::RHN - RedHat Network action type for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Action::PackageManager::RedHat::RHN;

use strict;

use PSGConf::Action::PackageManager::RedHat::RPMS
	qw(pkg_get_installed pkg_remove pkg_get_dependencies);

use PSGConf::Util;

our @ISA = qw(PSGConf::Action::PackageManager::RedHat::RPMS);


###############################################################################
###  pkg_get_latest_version() method
###############################################################################

sub pkg_get_latest_version
{
	my ($self, $psgconf, $pkg) = @_;
	my ($cmd, $version);
	my ($regex) = "\Q${pkg}\E\-([^-]+)-([^-]+)\\.[^\\.]*";

	### If it is a "special" virtual package (ie gpg key manangement)
	### skip the version and return undef
	return undef
		if ( $pkg eq 'gpg-pubkey' );

	if ( ! defined $self->{cache} ) {
		$cmd = $psgconf->data_obj('pkg_up2date_cmd')->get() . ' --nox --showall |';

		if (open (FP, $cmd)) {
			@{$self->{cache}} = <FP>;
          	close FP;
		} else {
			warn "\t* could not run up2date --nox --showall\n";
			return undef;
		}
	}

	### Now search through the cached versions.
	grep (/^${regex}$/ && ($version="$1-$2"), @{$self->{cache}});

	warn "\t* no updates found for package $pkg\n"
          if ( $psgconf->{verbose} && $version eq '' );

	return $version;
}


###############################################################################
###  pkg_install() method
###############################################################################

sub pkg_install
{
	my ($self, $psgconf, $pkg) = @_;
	my ($cmd, $res);

	###
	### Remove the version of the package so that up2date will install it.
	###
	($pkg) = $self->Parse($pkg);

	$cmd = $psgconf->data_obj('pkg_up2date_cmd')->get() . ' --nox --force --update ' . $pkg;

     $cmd .= '> /dev/null 2>&1'
          if ( ! $psgconf->{verbose});

	$res=&PSGConf::Util::RunCommand($cmd);

     print "\n"
          if ($psgconf->{verbose});

	return $res;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::PackageManager::RedHat::RHN - install RedHat packages

=head1 SYNOPSIS

  use PSGConf::Action::PackageManager::RedHat::RHN;

  $psgconf->register_actions(
		PSGConf::Action::PackageManager::RedHat::RHN->new(
			'quiet'		=> 1
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::PackageManager::RedHat::RHN> module provides 
a B<PSGConf> action class for updating RedHat packages from a RHN 
Server.

The B<PSGConf::Action::PackageManager::RedHat::RHN> class is derived 
from the B<PSGConf::Action::PackageManager::RedHat::RPMS> class, but it 
defines/overrides the following methods:

=over 4

=item I<pkg_get_latest_version($pkg)>

A method that will return a B<PSGConf::Data::String> that contains
the latest version of $pkg from the RHN repository.  Called from the 
B<PSGConf::Action::PackageManager::check()> method.

=item I<pkg_install($pkg)>

A method that will install a $pkg on the system.  Returns 0 for a
successfull install, non-zero for a failure.  Called from 
B<PSGConf::Action::PackageManager::do()> method.

=back

=head1 SEE ALSO

C<CE<lt>up2date(8)E<gt>>

L<perl>

L<PSGConf>

L<PSGConf::Action>

L<PSGConf::Action::PackageManager::RPMS>

L<PSGConf::Util>

=cut

