
###
###  Copyright 2000-2004 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PSGConf::Action::PackageManager::RedHat::Yum - Yum action type for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Action::PackageManager::RedHat::Yum;

use strict;

use PSGConf::Action::PackageManager::RedHat::RPMS
	qw(pkg_get_installed pkg_remove pkg_get_dependencies);

use PSGConf::Util;

our @ISA = qw(PSGConf::Action::PackageManager::RedHat::RPMS);


###############################################################################
###  pkg_get_latest_version() method
###############################################################################

sub pkg_get_latest_version
{
	my ($self, $psgconf, $pkg) = @_;
	my ($cmd, $version);
	my ($regex) = qr (\Q${pkg}\E\.([^\s\t\.]+)[\s\t]+(\d+:)?([^\s\t-]+)-([^\s\t-]+).*);

	### If it is a "special" virtual package (ie gpg key manangement)
	### skip the version and return undef
	return undef
		if ( $pkg eq 'gpg-pubkey' );

	if ( ! defined $self->{cache} ) {
		$cmd = $psgconf->data_obj('pkg_yum_cmd')->get() . ' list |';

		if (open (FP, $cmd)) {
			@{$self->{cache}} = <FP>;
          	close FP;
		} else {
			warn "\t* could not run $cmd\n";
			return undef;
		}
	}

	### Now search through the cached versions.
	grep (/^${regex}$/ && ($version="$3-$4"), @{$self->{cache}});

	warn "\t* no updates found for package $pkg\n"
		if ( $psgconf->{verbose} && $version eq '' );

	return $version;
}


###############################################################################
###  pkg_install() method
###############################################################################

sub pkg_install
{
	my ($self, $psgconf, $pkg) = @_;
	my ($cmd, $pkg2, $res);

	###
	### yum upgrade will not work if the package is not already
	### installed, so we have to determine if this package is an
	### upgrade or fresh install :(
	###
	($pkg2) = $self->Parse($pkg);
	$res=&PSGConf::Util::RunCommand(
		$psgconf->data_obj('pkg_rpm_cmd')->get() . ' --quiet -q ' . $pkg2 . '>/dev/null 2>&1', 1);

	$cmd = $psgconf->data_obj('pkg_yum_cmd')->get() 
		. ' -d ' . (! $psgconf->{verbose}? 0: 2)
		. ' -e 0 -y '
		. ($res? 'install ': 'upgrade ')
		. $pkg;

	$res=&PSGConf::Util::RunCommand($cmd);

     print "\n"
          if ($psgconf->{verbose});

	return $res;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::PackageManager::RedHat::Yum - install RedHat packages

=head1 SYNOPSIS

  use PSGConf::Action::PackageManager::RedHat::Yum;

  $psgconf->register_actions(
		PSGConf::Action::PackageManager::RedHat::Yum->new(
			'quiet'		=> 1
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::PackageManager::RedHat::Yum> module provides 
a B<PSGConf> action class for updating RedHat packages from a Yum 
Server.

The B<PSGConf::Action::PackageManager::RedHat::Yum> class is derived 
from the B<PSGConf::Action::PackageManager::RedHat::RPMS> class, but it 
defines/overrides the following methods:

=over 4

=item I<pkg_get_latest_version($pkg)>

A method that will return a B<PSGConf::Data::String> that contains
the latest version of $pkg from the Yum repository.  Called from the 
B<PSGConf::Action::PackageManager::check()> method.

=item I<pkg_install($pkg)>

A method that will install a $pkg on the system.  Returns 0 for a
successfull install, non-zero for a failure.  Called from 
B<PSGConf::Action::PackageManager::do()> method.

=back

=head1 SEE ALSO

C<CE<lt>yum(8)E<gt>>

L<perl>

L<PSGConf>

L<PSGConf::Action>

L<PSGConf::Action::PackageManager::RPMS>

L<PSGConf::Util>

=cut

