
###
###  Copyright 2000-2007 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PSGConf::Data::List - list data type for PSGConf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Data::List;

use strict;

use PSGConf::Data;

our @ISA = qw(PSGConf::Data);


###############################################################################
###  default is add() method
###############################################################################

sub default
{
	my ($self, $value) = @_;

	$self->add($value);
}


###############################################################################
###  unset method
###############################################################################

sub unset
{
	my ($self) = @_;

	$self->SUPER::set([]);
	return 1;
}


###############################################################################
###  set method
###############################################################################

sub set
{
	my ($self, $value) = @_;

#	print "==> List::set($value)\n";

	$self->unset();
	return $self->add($value);
}


###############################################################################
###  add method
###############################################################################

sub add
{
	my ($self, $value) = @_;

	$value = $self->_scalar_or_list($value);
	$self->set([])
		if (!defined($self->get()));

	die "non unique value specified '$value' for a unique list\n"
		if ( $self->{unique} && $self->find($value) );

	push(@{$self->get()}, @$value);

	return 1;
}


###############################################################################
###  add_top method
###############################################################################

sub add_top
{
	my ($self, $value) = @_;

	$value = $self->_scalar_or_list($value);
	$self->set([])
		if (!defined($self->get()));

	die "non unique value specified '$value' for a unique list\n"
		if ( $self->{unique} && $self->find($value) );

	unshift(@{$self->get()}, @$value);

	return 1;
}


###############################################################################
###  delete method
###############################################################################

sub delete
{
	my ($self, $value) = @_;
	my ($val);

	$value = $self->_scalar_or_list($value);
	$self->set([])
		if (!defined($self->get()));
	foreach $val (@$value)
	{
		$self->SUPER::set([ grep !/$val/, @{$self->get()} ]);
	}

	return 1;
}


###############################################################################
###  count method
###############################################################################

sub count
{
	my ($self) = @_;

	return scalar @{$self->get()};
}


###############################################################################
###  find method
###############################################################################

sub find
{
	my ($self, $value) = @_;
	my ($res);

	($res) = grep(/$value/, @{$self->get()});
	return ($res? $res: undef);
}


###############################################################################
###  replace method
###############################################################################

sub replace
{
	my ($self, $old, $new) = @_;
	my (@values, $i);

	@values = @{$self->get()};
	for ($i=0; $i < scalar @values; $i++) {
		$values[$i] =~ s/$old/$new/;
	}
	$self->SUPER::set([@values]);

	return 1;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Data::List - list data type class for PSGConf

=head1 SYNOPSIS

  use PSGConf::Data::List;

  $psgconf->register_data(
		'listobj'	=> PSGConf::Data::List->new()
	);

=head1 DESCRIPTION

The B<PSGConf::Data::List> module provides a class that
represents a list in an object so that it can be used with
B<PSGConf>.  Its methods can be used to manipulate the
encapsulated list from the B<PSGConf> data store(s).

The B<PSGConf::Data::List> class is derived from the
B<<PSGConf::Data> class, but it defines/overrides the
following methods:

=over 4

=item add()

Adds the supplied value to the end of the object's list.  The value can
be a scalar or a reference to a list, in which case the values in the
referenced list are added to the object's list.

If the object was created with the I<unique> attribute set then the
list values must be unique in this list.

=item unset()

Sets the object's value to an empty list.

=item set()

The same as add(), except that the existing list is emptied by calling
the unset() method before adding the new data.

=item default()

Calls the add() method.

=item add_top()

Same as add, but adds to the front of the list instead of the end.

=item delete()

Deletes elements from the list that match its argument.  Matching is
performed by using the argument as a regular expression.  The argument
can be a scalar or a reference to a list, in which case each item of the
referenced list is used to check the values in the object's list.

=item find()

Searches for value in the list, returns the count of items found in the list
or undef.

=item count()

Returns the number of elements in the list.

=item replace()

Searches for value1 in the list, and replaces it with value2 for all occurances.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Data>

=cut

